<?php
/**
 * RSS用 1.7拡張連載用
 *
 * @package
 *
 * @copyright 2008-2016 Shift Tech Inc. All Rights Reserved.
 * @author    シフトテック株式会社
 *
 * @version 1.0
 */
 require_once dirname(dirname(dirname(dirname(__FILE__)))).'/app_path.php';
 require_once APP_PATH.'/PcController.php';
 require_once LIB_PATH.'/common/pageView.php';

// empty
function _e($s)
{
    if(!isset($s) || is_null($s) || $s==="" || (is_array($s) && count($s) <= 0)) {
        return true;
    }

    return false;
}

/**
 * RSS用 1.7拡張連載用
 */
class Feed extends Model
{
    /**
     * RSS 1.0
     * @return [type] [description]
     */
    public function rss1()
    {
        $this->db = new DataBase();
        date_default_timezone_set('Asia/Tokyo');
        $data = $this->parseQuery();
        $xml = new SimpleXMLElement('<?xml version="1.0" encoding="UTF-8"?><rss xmlns:dc="http://purl.org/dc/elements/1.1/" xmlns:sy="http://purl.org/rss/1.0/modules/syndication/" version="2.0"><channel></channel></rss>');

        $res = $this->db->query('SELECT * FROM tbl_sitebase LIMIT 1');
        $site = $res->fetchRow(DB_FETCHMODE_ASSOC);
        if(_e($site)) {
            $this->error(500);
        }

        $name = $data['page'] ? " ".$data['page']['sidx_title'] : "";
        $xml->channel->addChild('title', $site['sbase_name'].$name);
        $xml->channel->addChild('link', $site['sbase_url']);
        $xml->channel->addChild('description', implode(' ', $data['cat_names']));
        $xml->channel->addChild('language', 'ja-jp');
        $xml->channel->addChild('pubDate', $this->getRssDate(date('Y-m-d H:i:s')));

        //ページ取得してくる
        $list = $this->getPages($data);

        if(is_array($list)) {
            foreach($list as $k=>$v) {
                $link = pageView::getSiteURL($v['sidx_dir'], $v['cat_dir'], $v['pidx_url'], $v['sidx_ssl']);
                if($v['pidx_directurl']!="") {
                    $link = $v['pidx_directurl'];
                }
                if($v['pidx_nolink'] == 1) {
                    $link = null;
                }
                $item = $xml->channel->addChild('item');
                $item->addChild('title', $v['pidx_title']);
                $item->addChild('link', $link);
                $item->addChild('description', htmlspecialchars($v['pidx_comment'], ENT_QUOTES, 'UTF-8'));
                $item->addChild('pubDate', $this->getRssDate($v['pidx_date']));
            }
        }

        $this->output($xml->asXML());
    }

    public function output($xml)
    {
        $ts = gmdate("D, d M Y H:i:s")." GMT";
        header("Expires: $ts");
        header("Last-Modified: $ts");
        header("Pragma: no-cache");
        header("Cache-Control: no-cache, must-revalidate");
        header("Content-Type: application/xml; charset=UTF-8");
        echo $xml;
    }

    public function getRssDate($date)
    {
        $format = DATE_RSS;
        return date($format, strtotime($date));
    }

    public function sanitize($str, $escape=false)
    {
        if($escape) {
            return "<[[CDATA\n".$str."\n]]>";
        }

        return "<[[CDATA\n".htmlspecialchars($str, ENT_QUOTES, 'UTF-8')."\n]]>";
    }

    public function getPages($data)
    {
        $values = array();
        $sql = 'SELECT
            p1.pidx_code,
            pidx_title,
            pidx_label,
            s1.sidx_code as cat_code,
            s2.sidx_code as sidx_code,
            s1.sidx_dir as cat_dir,
            s2.sidx_dir as sidx_dir,
            s1.sidx_title as cat_name,
            CASE
            WHEN s2.sidx_order IS NULL THEN s1.sidx_order
            ELSE s2.sidx_order END as menu_sidx_order,
            CASE
            WHEN s2.sidx_code IS NULL THEN s1.sidx_code
            ELSE s2.sidx_code END as menu_sidx_code,
            CASE
            WHEN s1.sidx_ssl=1 OR s2.sidx_ssl=1 THEN 1
            ELSE NULL END as sidx_ssl,
            pidx_date,
            pidx_event_date,
            pidx_event_edate,
            pidx_comment,
            pidx_url,
            pidx_comment_pic,
            pidx_comment_pic2,
            skin_path,
            pidx_directurl,
            pidx_directurl_flg,
            pidx_nolink

            FROM tbl_page_idx as p1

            JOIN tbl_section_idx as s1
            ON s1.sidx_code=p1.pidx_sidx_code

            LEFT JOIN tbl_section_idx as s2
            ON s1.sidx_parent_sidx_code=s2.sidx_code

            JOIN tbl_template
            ON s1.sidx_tpl_code = tpl_code

            JOIN tbl_skin
            ON tpl_skin_code = skin_code

            WHERE pidx_ptype=1
            AND pidx_active=1
            AND pidx_view IN(0,2)
            AND s1.sidx_active=1
            AND s1.sidx_view=2 AND (s2.sidx_view=2 OR s2.sidx_view IS NULL)
            AND (s1.sidx_extmodule_code IS NULL OR NOT(s1.sidx_extmodule_code >=1000 AND s1.sidx_extmodule_code < 2000)) ';
        //メニュー指定
        if($data['cat_codes'] && 0 < count($data['cat_codes'])) {
            $sql .= " AND s1.sidx_code IN(".$this->bind($data['cat_codes']).")";
            $values = array_merge($values, $data['cat_codes']);
        }
        //ラベル指定
        if(is_numeric($data['pidx_code']) && $data['rel_codes'] && 0 < count($data['rel_codes'])) {
            $sql .= ' AND (';
            foreach ($data['rel_codes'] as $k => $code) {
                if ($k > 0) {
                    $sql .= ' OR ';
                }
                $sql .= " pidx_label=?
                    OR pidx_label LIKE ?
                    OR pidx_label LIKE ?
                    OR pidx_label LIKE ? ";
                $values[] = $code;
                $values[] = "{$code}<>%";
                $values[] = "%<>{$code}<>%";
                $values[] = "%<>{$code}";
            }
            $sql .= ')';

            $sql .= " AND pidx_code <> {$data['pidx_code']} ";
        } elseif($data['label_codes'] && 0 < count($data['label_codes'])) {
            $sql .= ' AND (';
            foreach ($data['label_codes'] as $k => $code) {
                if ($k > 0) {
                    $sql .= ' OR ';
                }
                $sql .= " pidx_label=?
                    OR pidx_label LIKE ?
                    OR pidx_label LIKE ?
                    OR pidx_label LIKE ? ";
                $values[] = $code;
                $values[] = "{$code}<>%";
                $values[] = "%<>{$code}<>%";
                $values[] = "%<>{$code}";
            }
            $sql .= ')';
        }

        $nowdate = date('Y-m-d H:i:s');
        $sql .= "  AND pidx_date <= '{$nowdate}' AND
            (pidx_edate Is Null OR pidx_edate >= '{$nowdate}') ";

        $sql .= "ORDER BY p1.pidx_order desc, {$data['order']} p1.pidx_sameday_priority {$data['order1']}, menu_sidx_order {$data['order1']}, menu_sidx_code {$data['order1']}, s2.sidx_order {$data['order1']}, s2.sidx_code {$data['order1']}, s1.sidx_order {$data['order1']}, s1.sidx_code {$data['order1']}, pidx_code {$data['order2']} ";

        if(is_numeric($data['limit'])) {
            if(!is_numeric($data['offset'])) {
                $data['offset'] = 0;
            }
            $sql .= " LIMIT ? OFFSET ?";
            $values[] = intval($data['limit']);
            $values[] = intval($data['offset']);
        } else if(is_numeric($data['offset'])) {
            $sql .= " LIMIT 100 OFFSET ?";
            $values[] = intval($data['offset']);
        }


        $res = $this->db->prepare($sql, $values);
        $list = array();
        if($res) {
            while($row = $res->fetchRow(DB_FETCHMODE_ASSOC)) {
                $list[] = $row;
            }
        }

        return $list;
    }

    public function parseQuery()
    {
        $sidx_code = $this->getRequest('s');
        $cat_code = $this->getRequest('c');
        $pidx_code = $this->getRequest('pidx');
        $order_no = $this->getRequest('o');
        $dir = $this->getRequest('d');
        $label = $this->getRequest('l');
        $rel = $this->getRequest('r');
        $prev = $this->getRequest('prev');
        $limit = $this->getRequest('limit');
        $offset = $this->getRequest('offset');

        if(!_e($sidx_code) && !is_numeric($sidx_code)) {
            $this->error(400);
        }
        if(!is_null($cat_code) && !is_numeric($cat_code) && !is_array($cat_code)){
            $this->error(400);
        }
        if(!is_null($label) && !is_numeric($label) && !is_array($label)){
            $this->error(400);
        }
        if(_e($order_no) || !is_numeric($order_no)) {
            $this->error(400);
        }
        if(!_e($limit) && !is_numeric($limit)) {
            $this->error(400);
        }
        if(!_e($offset) && !is_numeric($offset)) {
            $this->error(400);
        }
        if(!_e($prev) && $prev == 1) {
            $prev = true;
        }

        //対象ページ。
        $res = $this->db->prepare('SELECT * FROM tbl_section_idx WHERE sidx_active=1 AND sidx_archive = 0 AND sidx_view = 2 AND sidx_code = ?', array($sidx_code));
        //nullの時は全メニュー対象
        $page = $res->fetchRow(DB_FETCHMODE_ASSOC);

        if(is_array($cat_code)) {
            foreach($cat_code as $c) {
                if(!is_numeric($c)) {
                    unset($cat_code[$k]);
                }
            }
        }
        if(is_numeric($cat_code)) {
            $cat_code = array($cat_code);
        }
        $pages = array();
        $cat_names = array();
        $cat_codes = array();
        if(0 < count($label)) {
            $res = $this->db->prepare('SELECT * FROM tbl_section_idx WHERE sidx_active = 1 AND sidx_archive = 0 AND sidx_view = 2 AND sidx_code IN ('.$this->bind($cat_code).')', $cat_code);
            if($res) {
                while($row = $res->fetchRow(DB_FETCHMODE_ASSOC)) {
                    $pages[] = $row;
                    $cat_names[] = $row['sidx_title'];
                    $cat_codes[] = $row['sidx_code'];
                }
            }
        }

        //ラベル
        if(is_array($label)) {
            foreach($label as $k=>$c) {
                if(!is_numeric($c)) {
                    unset($label[$k]);
                }
            }
        }
        if(is_numeric($label)) {
            $label = array($label);
        }

        $labels = array();
        $label_codes = array();
        if(0 < count($label)) {
            $res = $this->db->prepare('SELECT * FROM tbl_label WHERE label_active = 1 AND label_admin_type = 0 AND label_code IN ('.$this->bind($label).')', $label);
            if($res) {
                while($row = $res->fetchRow(DB_FETCHMODE_ASSOC)) {
                    $labels[] = $row;
                    $label_codes[] = $row['label_code'];
                }
            }
        }

        //関連ラベルの時
        $rel_codes = array();
        if(is_numeric($pidx_code) && isset($_GET['r']) && preg_match('/^[\d,]+$/', $rel)) {
            $rel_codes = explode(',', $rel);
        }

        $order = "";
        $order1 = " ASC";
        $order2 = " DESC";

        switch(intval($order_no)) {
            case 1:
                $order = "p1.pidx_date ASC,";
            break;
            case 2:
                $order = "p1.pidx_date DESC,";
                $order1 = " DESC";
                $order2 = " ASC";
            break;
            case 3:
                $order = "(CASE WHEN p1.pidx_event_date IS NULL THEN '9999-01-01' ELSE pidx_event_date END) ASC,";
            break;
            case 4:
                $order = "(CASE WHEN p1.pidx_event_date IS NULL THEN '9999-01-01' ELSE pidx_event_date END) DESC,";
                $order1 = " DESC";
                $order2 = " ASC";
            break;
            case 5:
                $order = "p1.pidx_title ASC,";
            break;
            case 6:
                $order = "p1.pidx_title DESC,";
                $order1 = " DESC";
                $order2 = " ASC";
            break;
        }

        return compact('page','pages','pidx_code','cat_names','cat_codes','labels','label_codes','rel_codes','order','order1','order2','sidx_code','cat_code','order_no','dir','label','rel','prev','limit','offset');
    }

    public function bind($array)
    {
        return implode(",",array_pad(array() ,count($array), '?'));
    }

    public function error($code=400)
    {
        switch($code) {
            case 500:
                header('HTTP/1.0 500');
                die('500 Internal Server Error');
                break;
            case 400:
            default:
                header('HTTP/1.0 400');
                die('400 Bad Request');
                break;
        }
        exit;
    }
}

$feed = new Feed();
$feed->rss1();
exit;
